﻿using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Input;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using GalaSoft.MvvmLight.Messaging;
using Windows.Networking.Proximity;
using Wp7nl.Devices;

namespace TtcDemo.Viewmodels
{
  public class NfcConnectViewModel : ViewModelBase
  {
    public NfcConnectViewModel()
    {
      Init();
    }

    private TtcSocketHelper ttcSocketHelper;

    public ObservableCollection<string> ConnectMessages { get; private set; }
    public ObservableCollection<string> ReceivedMessages { get; private set; }

    private bool canInitiateConnect;
    public bool CanInitiateConnect
    {
      get { return canInitiateConnect; }
      set
      {
        if (canInitiateConnect != value)
        {
          canInitiateConnect = value;
          RaisePropertyChanged(() => CanInitiateConnect);
        }
      }
    }

    private bool isConnecting;
    public bool IsConnecting
    {
      get { return isConnecting; }
      set
      {
        if (isConnecting != value)
        {
          isConnecting = value;
          RaisePropertyChanged(() => IsConnecting);
        }
      }
    }

    private bool canSend;
    public bool CanSend
    {
      get { return canSend; }
      set
      {
        if (canSend != value)
        {
          canSend = value;
          RaisePropertyChanged(() => CanSend);
        }
      }
    }

    private string message;
    public string Message
    {
      get { return message; }
      set
      {
        if (message != value)
        {
          message = value;
          RaisePropertyChanged(() => Message);
        }
      }
    }

    public ICommand StartCommmand
    {
      get
      {
        return new RelayCommand(
            () =>
            {
              ConnectMessages.Add("Connect started...");
              CanSend = false;
              CanInitiateConnect = false;
              ttcSocketHelper.Start();
            });
      }
    }

    public ICommand ResetCommand
    {
      get
      {
        return new RelayCommand(
            () =>
            {
              ConnectMessages.Clear();
              CanInitiateConnect = true;
              IsConnecting = true;
              CanSend = false;
              ttcSocketHelper.Reset();
            });
      }
    }

    public ICommand SendComand
    {
      get
      {
        return new RelayCommand(
          () =>
          {
            ttcSocketHelper.SendMessage(Message);
            Message = string.Empty;
          }
       );
      }
    }

    private void Init()
    {
      Messenger.Default.Register<NavigationMessage>(this, ProcessNavigationMessage);

      if (ConnectMessages == null)
      {
        ConnectMessages = new ObservableCollection<string>();
      }

      if (ReceivedMessages == null)
      {
        ReceivedMessages = new ObservableCollection<string>();
      }

      if (!IsInDesignMode)
      {
        if (ttcSocketHelper == null)
        {
          ttcSocketHelper = new TtcSocketHelper();
          ttcSocketHelper.ConnectionStatusChanged += ConnectionStatusChanged;
          ttcSocketHelper.MessageReceived += TtsHelperMessageReceived;
        }
        else
        {
          CanSend = true;          
        }
      }
      IsConnecting = true;
    }

    private void ProcessNavigationMessage(NavigationMessage message)
    {
      CanInitiateConnect = !message.IsStartedByNfcRequest;
    }

    private void TtsHelperMessageReceived(object sender,
      ReceivedMessageEventArgs e)
    {
      Deployment.Current.Dispatcher.BeginInvoke(() =>
        ReceivedMessages.Add(e.Message));
    }

    private void ConnectionStatusChanged(object sender,
      TriggeredConnectionStateChangedEventArgs e)
    {
      Deployment.Current.Dispatcher.BeginInvoke(() =>
        ConnectMessages.Add(GetMessageForStatus(e.State)));
      if (e.State == TriggeredConnectState.Completed)
      {
        Deployment.Current.Dispatcher.BeginInvoke(() =>
        {
          IsConnecting = false;
          CanSend = true;
        });
      }
    }

    private static string GetMessageForStatus(TriggeredConnectState state)
    {
      switch (state)
      {
        case TriggeredConnectState.Listening:
          return "Listening....";

        case TriggeredConnectState.PeerFound:
          return "Opponent found";

        case TriggeredConnectState.Connecting:
          return "Opponent found";

        case TriggeredConnectState.Completed:
          return "Connection succesfull!";

        case TriggeredConnectState.Canceled:
          return "Connection canceled";

        default: //TriggeredConnectState.Failed:    
          return "Connection failed";
      }
    }
  }
}
